/* 28 March 2001. Daniel Hellerstein (danielh@crosslink.net)
                 DirTree ver 1.11 
  Sort (and display) all files in a directory tree by filename.
  Also supports directory summary capabilities.

   You may freely use this utility,
   You may also find the 3 procedures used here to be useful.
   These are:
        resolve_filename: return a fully qualified filename, given                    
                          a relative filename and directory information
        StemSort: sort a stem variable (using a bubble sort)
        dir_exists: detects whether a directory exists

Standard disclaimer:
  This program, and it's included procedures, may be
  freely used, but you use them at your own risk.

For usage instructions, enter DIRTREE ? at an os/2 command prompt.

*/

/***** -- User changeable parameters    ****/

/* default output file (if -FILE is given with no argument ) */
outfile='DIRTREE.LST'

/* used as a filler when the file name has not changed */
filler=' "" '

/* program string for displaying html output */
vu_prog='NETSCAPE -l en '


/* character used to indicate depth of subdirectory
   (used with -DIRSUMMARY option */
subflag='$'

/**** End of user changeable parameters  ****/


parse arg  aa
parse var aa dirfile '-' astuff
if astuff<>'' then astuff='-'||astuff

if dirfile='' then dirfile='*'

if aa='?' | strip(aa)='-?' | strip(aa)='/?'  then do
say 'DirTree: List files in a directory tree -- sorted by name, size, and/or date. '
say
say "Usage: x:>DIRTREE directory\file_pattern [-options]  "
say "  Where:    directory = directory tree to list (default is current directory) "
say "         file_pattern = a file pattern (* is the default)"
say "       or, you can use dir1\file1 dir2\file2 ... to list multiple 'trees' "
say "  Options include:"
say '      -SUMMARY =  Just display  a summary (#files/#bytes).       '
say "      -Onxds   =  Sort by n(ame), x(tension),d(ate), and/or s(ize). "
say "                  The  default is to sort by name only."
say "      -D       =  Decending sort (default is ascending). "
say " "
say " To list all options, enter DIRTREE ?? "
say 
say "Examples: x:>dirtree \archive1\*.zip -Ond -D "
say "          x:>dirtree  -summary "
say "          x:>dirtree  f:\archive1\*.zip e:\projects\* -SHORT -FILEONLY A.LST "
say '          x:>dirtree  "pets\My Cat*.*" -SHORT -FILEONLY A.LST '
say "          x:>dirtree  * -x  \OLD \ARCHIVE "
say "Or, enter DIRTREE at a command prompt (with no arguments) to be prompted. "
exit
end

if dirfile='??' then do
say "  Options (DirTree ver 1.11):"
say '   -CT          = Count of files with this "filename"'
say "   -D           = Decending sort (default is ascending) "
say '   -DIRSUMMARY = Display summary of each directory '
say '                 (in dir: #files/#bytes; in subtree: subdirs/files/bytes) '
say '   -FILE filename.ext = Write results to filename.ext '
say '   -FILEONLY file.ext = same as -FILE, and do not display results on screen'
say '   -HTML file.ext     = Write results using HTML formatting         '
say '   -HTML_VU file.ext  = Same as -HTML, and run Netscape to view file.ext    '
say "   -Onxds       = Sort by n(ame), x(tension), d(ate), and/or s(ize) "
say '   -SHORT,-LONG,-VERYLONG = Style of display '
say '   -SUMMARY     = Display  summary (#files/#bytes); '
say '                  (do NOT display matching files)'
say '   -X  xstr     = Exclude files that contain xstr in their fully qualifed name'
say '                  Xstr can contain several space delimited substrings'
say '  *:filename    = Search ALL drives for filename '
say '  LOCAL:filename    = Search ALL LOCAL drives for filename '
say '  REMOTE:filename   = Search ALL REMOTE (LAN) drives for filename '



exit
end /* do */
say

/*---   Load REXX libraries ----- */
/* Load up advanced REXX functions */
foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  call SysLoadFuncs
end

foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end
gotrexxlib=rxfuncquery('rexxlibregister')

sortorder=''
isdescend=0
docount=0
dolong=0
lmxdate=14
dosummary=0
dodirsummary=0
toscreen=1
tofile=0
nbdd=0
excstring=''
dohtml=0
htmlview=0

if dirfile<>'' then do 
  if dirfile='*' then dirfile='*.*'

  if pos('"',dirfile)=0 then do
      dirfile=space(dirfile)
      dirfile=translate(dirfile,'+',' ')
      bdd=dirfile
  end 
  else do               /* parse out " xxx " stuff */
     t1=dirfile
     bdd=''
     do forever
       if t1='' then leave
       parse var t1 a1 '"' a2 '"' t1
       if  a1<>'' then do
          bdd=bdd'+'strip(a1)
       end
       if a2<>'' then do
          bdd=bdd'+'strip(a2)
       end 
     end
  end 
  bdd=strip(bdd,'l','+')
  do until bdd=''

      parse var bdd dirfile '+' bdd
      dirf=translate(dirfile)
/* check for *: syntax, IF found, expand */
     if abbrev(strip(dirf),'*:')=1 | ,
       abbrev(strip(dirf),'LOCAL:')=1 | ,
        abbrev(strip(dirf),'REMOTE:')=1 then do
        parse var dirfile XX ':' dirfile1
        XX=translate(STRIP(XX))
        IF XX='*' then XX='USED'
        oy=sysdrivemap(,XX)
        do until oy=''
           parse var oy aa oy
           bdd=strip(aa)||dirfile1||'+'||bdd
        end /* do */
        iterate
     end 
      nbdd=nbdd+1
     adrive=''
     if pos(':',dirfile)>0 then do
           parse var dirfile adrive ':' dirfile
           adrive=adrive':'
     end 
     dirfile=translate(dirfile,'\','/')
     apath=''
     aname=dirfile
     ii=lastpos('\',dirfile)
     if ii>0 then do
         apath=left(dirfile,ii)
         aname=substr(dirfile,ii+1)
     end 
     if aname='' then aname='*.*'
     filename.nbdd=aname
     adir.nbdd=strip(adrive||apath)

  end

  astuff=translate(astuff)||' '
  if pos('-DIR',astuff)>0 then dodirsummary=1
  if pos('-D ',astuff)>0 then isdescend=1
  if pos('-CT',astuff)>0 then docount=1
  if pos('-LONG',astuff)>0 then dolong=1
  if pos('-VERYLONG',astuff)>0 then dolong=2
  if pos('SHORT',astuff)>0  then dolong=-1
  if pos('-SUM',astuff)>0 then dosummary=1
  if pos('-X',astuff)>0 then do
    parse upper var astuff . '-X ' excstring '-' astuff ; excstring=strip(excstring)
  end

  if pos('-HTML',astuff)>0 then do
       dohtml=1
       tofile=1
       toscreen=0
       parse upper var astuff . '-HTML' a23 '-'  . ; parse var a23 a2 a3 .
       if right(strip(a2),2)='VU' |abbrev(strip(a2),'VIEW')=1 then do
          htmlview=1
          a2=a3
       end 
       a2=strip(a2)
       if a2='' then a2=outfile
       if stream(a2,'c','query exists')=' ' then do
           say "Writing HTML results to "a2
       end
       else do
          say "OverWriting HTML results to: "a2
          foo=sysfiledelete(a2)
       end
       foo=stream(a2,'c','open write')
       if foo<>'READY:' then do
                say "Sorry, can not write to: "a2
                exit
       end 
       outfile=a2
       call ini_html    /* initialize html stuff in outfile */
  end 


  if pos('-FILE',astuff)>0 then do
       tofile=1
       parse upper var astuff . '-FILE' a23 '-'  . ; parse var a23 a2 a3 .
       if strip(a2)='ONLY' then do
          toscreen=0
          a2=a3
       end 
       a2=strip(a2)
       if a2='' then a2=outfile
       if stream(a2,'c','query exists')=' ' then do
           say "Writing results to "a2
       end
       else do
          say "OverWriting results to: "a2
          foo=sysfiledelete(a2)
       end
       foo=stream(a2,'c','open write')
       if foo<>'READY:' then do
                say "Sorry, can not write to: "a2
                exit
       end 
       outfile=a2
  end 

  if dolong=-1  then do
    filler='    '
    lmxdate=8
  end
  parse var astuff . '-O' sortorder .
 
  do yy=1 to nbdd
    adir.yy=resolve_filename('',adir.yy,,1)
    if adir.yy='' then do
       say "No such directory: "adir.yy
       exit
    end 
  end
  if nbdd=1 then do
       call dosay "Searching in: "adir.1
       call dosay "         for: "filename.1
   end
   else do
     do yy=1 to nbdd
       call dosay "Searching in: "adir.yy
       call dosay "         for: "filename.yy
     end
  end /* do */
   if excstring<>'' then do
      do www=1 to words(excstring)
         if www=1 then
            call dosay "Excluding files that match: *"||strip(word(excstring,www))||"*"
        else
            call dosay "                          : *"||strip(word(excstring,www))||"*"
      end
    end
end 

else do                        /* from keyboard */
  say "DirTree: sort files by names across a directory tree."
  call charout," Starting directory: "
  parse pull adir
  adir=resolve_filename('',adir,,1)
  if adir='' then do
     say "No such directory: "adir
     exit
  end 
  call charout, " Files to find (default=*): "
  parse pull filename
  if filename='' then filename='*'

  adir.1=adir
  filename.1=filename
  nbdd=1

  call charout,' Summary stats only (Y=yes)?'
  pull dosummary
  if dosummary<>'Y' then do
     call charout,' Sort order string; d=date, s=size,n=name, x=ext (i.e.; nd or ds):'
     pull sortorder
     call charout,' Descending order (1), or Ascending (ENTER) :'
     pull isdescend
  end
  else do
       dosummary=1
  end 
end

/* ready to rumble */

do jj=1 to nbdd
  if dir_exists(adir.jj)=0 then do
      say "% No such directory: "adir.jj
       exit
  end 
end

if dosummary=1 | dodirsummary=1  then do               /* just # files, # bytes, and # dirs? */
   dohtml=0                             /* html only for file listing */
   foo=dir_summary(nbdd)
   signal alldone
end 


if sortorder='' then sortorder='N'
fils.=0
iat=0
do mm=1 to nbdd
  oo=sysfiletree(adir.mm||filename.mm,'tfils.','FTS')
  do iuu=1 to tfils.0
     iat=iat+1
     fils.iat=tfils.iuu
  end /* do */
  fils.0=iat
end

/* grab the files */
lmxname=0; lmxsize=0 ;mm=0
do mm0=1 to fils.0
   mm=mm+1
   parse var fils.mm0  date.mm size.mm . aa
   if excstring<>'' then do
     do www=1 to words(excstring)
         excstring1=strip(word(excstring,www))
         if pos(excstring1,translate(aa))>0 then do
             mm=mm-1
            iterate mm0
         end
      end
   end
   nam.mm=translate(filespec("n",aa)); path.mm=filespec('p',aa)
   if nbdd>1 | dohtml=1 then path.mm=translate(filespec("D",aa))||path.mm
   lmxsize=max(length(size.mm),lmxsize)
   lmxname=max(length(nam.mm),lmxname)
   if left(date.mm,2)<80 then
      date.mm='20'||date.mm
   else
     date.mm='19'||date.mm
end /* do */
call dosay "# of matching files= " mm

srtx.=''
kk=0
do ll2=1 to mm
  do ll=1 to length(sortorder)
     atype=substr(sortorder,ll,1)
     
     select
        when atype='N' then do
            srtx.ll2=srtx.ll2||left(nam.ll2,lmxname+1,' ')
        end
        when atype='X' then do
            parse upper var nam.ll2 . '.' anextx .
            srtx.ll2=srtx.ll2||left(anextx,lmxname+1,' ')
        end
        when atype='S' then do
            srtx.ll2=srtx.ll2||right(size.ll2,lmxsize+1,'0')
        end
        when atype='D' then do
            srtx.ll2=srtx.ll2||left(date.ll2,15,' ')
        end
        otherwise do
            nop
        end
     end  /* select */
  end /* do */
  srtx.ll2=srtx.ll2||right(ll2,7,' ')
end /* do */

srtx.0=mm
if srtx.0>0 then do             /* any matches? */
  call dosay "Sorting by " sortorder " ... "
  if dohtml=1 then do 
     call lineout outfile,'</pre>'
    do igoo=1 to 26
       alet=d2c(igoo+64)
       call lineout outfile,'<a href="#'||alet||'">'Alet'</a>&nbsp;'
    end /* do */
  
    call lineout outfile,'<br><ul>' 
  end /* do */
  

  /* use rexxlib sort, if available */
  if gotrexxlib=0 then do
     foo=arraysort('srtx',,,,,,'I')
  end 
  else do
    call stemsort 'srtx.',1 
  end
end

oldname='..'
oldsize='..'
olddate='..'
oldpath='..'
oldext='..'

j1=1 ; j2=srtx.0 ; j3=1
if isdescend=1  then do
   j1=srtx.0; j2=1; j3=-1
end /* do */

oldlet=''

/* Now write the output, using filename, date or time (optional), and path */
ict=0
do mm=j1 to j2 by j3
   ith=strip(word(srtx.mm,words(srtx.mm)))
   a2=left(nam.ith,lmxname)

   if a2=oldname then do                        /*always list filename */
       mold=1
       a2=left(filler,lmxname,' ')
       ict=ict+1
   end
   else do                      /* new name */
       mold=0
       oldname=a2
       if docount=1 then call dosay '('ict') 'tmpline
       ict=1
    end
    if dohtml=1 & mold=0 then do
        a2='<li><a href="file:///'||strip(path.ith||nam.ith)||'" target="viewer" '
        a2=a2||'>'||strip(nam.ith)||'</a>&nbsp;&nbsp;&nbsp;'
        alet=left(strip(nam.ith),1)
           
      if alet<>oldlet then do
           if verify(alet,'ABCDEFGHIJKLMNOPQRTSUVWXYZ')=0 then do
             a2='<a name="'||translate(alet)||'">'||a2||'</a>'
             oldlet=alet
          end
      end


/* might do something with this later
        a2='<li><a href="file:///'||strip(path.ith||nam.ith)||'" target="viewer" '
        foof="'"||nam.ith||"'"
        a2=a2||' onMouseOver="window.status='foof'; return true ;"'
        a2=a2||'>'||strip(nam.ith)||'</a>&nbsp;&nbsp;&nbsp;'
*/
    end 
    if dohtml=1 & mold=1 then do
        a2='<br>&nbsp;&nbsp;&nbsp;&nbsp;<a href="file:///'||strip(path.ith||nam.ith)||'" target="viewer">'||strip(path.ith)||'</a>'
    end 
   

   if pos('S',sortorder)>0 then do                      /* size is optional */
      if oldsize<>size.ith then do
        a3=right(strip(size.ith,'l','0'),lmxsize)
        oldsize=size.ith
      end
      else do
        a3=right(filler,lmxsize)
      end
      a2=a2||' '||a3
   end
   if pos('D',sortorder)>0 then do                      /*date is optional */
      if olddate<>date.ith then do
        a3=substr(date.ith,3)
        parse var a3 yr '/' mo '/' day '/' hr '/' min
        a3=yr'-'mo'-'day
        if dolong<>-1 then a3=a3||' 'hr':'min
        olddate=date.ith
      end
      else do
        a3=right(filler,lmxdate)
      end
      a2=a2||' '||a3
   end
   
   if dolong=-1 & ict>1 & length(path.ith)>length(oldpath) then do         /* shorten path? */
      icc=compare(translate(oldpath),translate(path.ith))
      icc2=max(2,lastpos('\',oldpath,icc))
      showpath=copies(' ',icc2-1)||substr(path.ith,icc2)
      oldpath=path.ith
   end
   else do
       oldpath=path.ith
       showpath=path.ith
   end /* do */
        
   select
     when dolong=2 then do  
         a2=a2||' '||date.ith||'   '||left(size.ith,10)
        if dohtml=0 then a2=a2||' 'path.ith||nam.ith

     end /* do */
     when dolong=1 & dohtml=0 then a2=a2||' '||path.ith||nam.ith
     when (dolong=0 | dolong=1) & dohtml=1 then nop
     otherwise a2=a2||' 'showpath
   end
   
/* either write line, or retain (and write with count later) */

   if docount=1 then do
      if ict=1 then tmpline=a2
   end
   else do
      call dosay a2 
   end
end /* do */
if docount=1 then call dosay '('ict') 'tmpline

if dohtml=1 then do
   call lineout outfile,'</ul></body></html>'
end 

alldone:
if tofile=1 then call lineout outfile
if htmlview=1 then do
   goo=resolve_filename(outfile,,,)
    foo=vu_prog' file:///'goo
      '@start /f 'foo
      say " >>> view "goo " with " vu_prog
      say"       (it might take a few seconds)"
end

exit


/*     ************************************************               */
/* the following procedures might prove useful in other contexts....  */
/*      *************************************************              */

/* this simple sort procedure is courtesy of Stan Irish, and
was obtained from comp.lang.rexx
Usage:
   call StemSort 'stemname.',column
where
    stemname = The name of a stem variable containing an "array" to
               sort.   
                  stemname.0 MUST be set to the number of elements
                  in the array!
    column   = (optional) the column number (the character number of
               values in (stemname.) to sort from. 
               If not specified, sort from column 1.

No value is returned, but 'stemname.' is sorted in place.

*/


StemSort:
  !stem = arg(1)
  call StemSortProc !stem,arg(2)
  return 0

StemSortProc:Procedure expose (!stem)
/* returns:  nothing
 Uses:     xxx = value(stemname.i) to get element values
       and  rc  = value(stemname.i,xxx) to set element values
*/

  sortstem = arg(1)
  If datatype(arg(2)) = 'NUM' then SortColumn = arg(2)
  Else SortColumn = 1

  d = value(sortstem||0) % 2              /* d is a distancemeasurement     */
  do while d > 0
    do until finished             /* start of mini-bubblesort loop   */
      finished = 1
      do i=1 to value(sortstem||0)-d
        j = i+d           /* we now compare and swap items i and i+d */
        if substr(value(sortstem||i),SortColumn) >substr(value(sortstem||j),SortColumn) then
          do
            temp = value(sortstem||i)
            rc = value(sortstem||i,value(sortstem||j))
            rc = value(sortstem||j,temp)
            finished = 0
          end
      end
    end                           /* end of mini-bubblesort loop     */
    d = d%2
  end
  RETURN ''


/* --------------------------------------------------------------------*/
/* Resolve a filename into a fully qualified file.
   This will take  a variety of filenames; including such forms as:
   FOO.BAR, E:FOO.BAR,  XYZ\FOO.BAR, and E:ABC\FOO.BAR

   Returns the fully qualified filename; or (if nocheck<>1,
   a '' if this filename does not exist.

Usage:
  filename=resolve_filename(a_filename,a_directory,default_ext,nocheck)
where
  a_filename = a filename  to use
               a_filename can contain "path information". If this
               is relative path information, then the path information
               from a_filename will be appended to the a_directory.
 a_directory = a directory, or a relative directory, to use.
               If a relative directory, a_directory will be converted
               to a fully qualified directory before path information
               from a_filename is appended.
 default_ext = add this extension to a_filename, if a_filename does not
               have a period (a .) in it
    nocheck  = If 1, do NOT verify the existence of this file
and
  filename   = a fully qualified filename, or a '' (signifying "no such
               file)


Hint: if you do not specify a filename, then resolve_filename will
      check for the existence of a directory (rather then an explicit
      file within the directory
*/

resolve_filename:procedure

parse arg afile,adir,defext,nocheck
afile=strip(afile) ; adir=strip(adir)

curdir0=directory()
curdir=curdir0'\'

if adir='' then adir=curdir     /* no adir specified, use current */

if right(adir,1)<>'\' & right(adir,1)<>':' then adir=adir'\'

usedrive=filespec('D',adir)
usedrive0=usedrive

if usedrive='' then usedrive=filespec('D',curdir) /* no drive in adir, use current*/

usepath=filespec('P',adir)
if left(usepath,1)<>'\' then do    /* relative to current usedrive path */
   foo=directory(usedrive)'\'
   foo2=directory(curdir0)
   usepath=filespec('p',foo)||usepath
end /* do */
oldfile=filespec('n',afile)

/* a hack, but what the heck.. */
do forever
  if pos('\\',usepath)=0 then leave
  parse var usepath a1 '\\' a2
  if length(a1)=0 then 
     usepath='\'
  else
     usepath=a1'\'
  if a2='' then leave
  usepath=usepath||a2
end
select
  when substr(afile,2,2)=":\" then do /* if 2-3 = :\, then use afile as is */
     usefile=afile
  end /* do */

  when substr(afile,2,1)=':' then do    /* relative file name on drive */
       
      if usedrive0='' then do            /* perhaps use usepath? */
          usefile=left(afile,2)||usepath||oldfile
      end               /* otherwise, use afile as is */
      else do
         usefile=afile
      end /* do */
  end
  when left(afile,1,1)='\' then do      /* attach adir drive */
      usefile=usedrive||afile
  end
  otherwise do
      usefile=usedrive||usepath||afile
  end
end

if pos('.',afile)=0 & defext<>'' then usefile=usefile||'.'||strip(defext,'l','.')

/* a hack, but what the heck.. */
do forever
  if pos('\\',usefile)=0 then leave
  parse var usefile a1 '\\' a2
  usefile=a1'\'
  if a2='' then leave
  usefile=a1||a2
end

if nocheck=1 then return usefile

if afile='' then do                     /* check for existence of directory*/
   isit=dir_exists(afile)
   if isit=0 then return ''
   return afile
end

file=stream(usefile,'c','query exists')  /* check for existence of a file */
return usefile


/*************************************************/
/* Check for the existence of a directory. Correctly identifies
   empty directories.
Usage:
   flag=dir_exists(a_directory)
where
   flag=1 if a_directory exists (it might be an empty directory )
   flag=0 if it doesn't exist
*/
dir_exists:procedure 
parse arg adir

adir=strip(adir)
adir=strip(adir,'t','\')
nowdir=directory()
nowdrive=filespec('d',nowdir'\')
nowpath=filespec('p',nowdir'\')
adr=filespec('d',adir)
if adr='' then do
   if abbrev(adir,'\')=0 then 
       adir=nowdrive||nowpath||adir
   else
       adir=nowdrive||adir
end /* do */

foo=sysfiletree(adir,goo,'D')
if  goo.0>0  then return 1
return 0

/* summary of this directory tree */
dir_summary:procedure expose filename. adir. toscreen tofile outfile excstring dodirsummary subflag
parse arg ndo
totfils=0
totdirs=0
totsize=0
numeric digits 12
dirstats.=0
do mm=1 to ndo
   todo=adir.mm||filename.mm
   oo=sysfiletree(todo,'fils','BTS')
   aline='>> '||translate(todo)
   if length(todo)>40 then do
      call dosay aline
      aline=''
   end
   nfiles=0 ; nsize=0
   isz=0
   do m=1 to fils.0
      isdir=0
      attribs=translate(strip(word(fils.m,3)))
      fname=translate(word(fils.m,4))
      if fname='.' | fname='..' then iterate
      if excstring<>'' then do
        do www=1 to words(excstring)
           excstring1=strip(word(excstring,www))
           if pos(excstring1,translate(fname))>0 then iterate m
        end
/*         if pos(excstring,translate(fname))>0 then iterate */
      end 
      if substr(attribs,2,1)='D' then isdir=1   /* this is a directory */
      thissize=word(fils.m,2)
      if dodirsummary=1  then do
         dname=filespec('d',fname)||filespec('p',fname)
         if dirstats.dname.!x=0 then do
            ii=dirstats.0+1
            dirstats.ii=dname
            dirstats.0=ii
            dirstats.!maxchar=max(dirstats.!maxchar,length(dname))
            if isdir=0 then do 
               dirstats.dname.!ct=1
               dirstats.dname.!size=thissize
            end
            dirstats.dname.!x=1
         end /* do */
         else do
            if isdir=0 then do
                dirstats.dname.!size=thissize+dirstats.dname.!size
                dirstats.dname.!ct=1+dirstats.dname.!ct
            end
         end
      end
      if isdir=0 then do
        isz=isz+thissize
        nfiles=nfiles+1
     end
    end

    aline=aline||" : # files=" nfiles
    aline=aline||" ("||addcomma(isz)||")"
    totsize=totsize+isz
    totfils=totfils+nfiles

  oo=sysfiletree(adir.mm,'fils2','DTS')
  if excstring='' then do
     ndirs=fils2.0
  end
  else do
     ndirs=0
      do ll=1 to fils2.0
        do www=1 to words(excstring)
           excstring1=strip(word(excstring,www))
           if pos(excstring1,translate(fils2.ll))>0 leave ll
         end
         ndirs=ndirs+1
      end               /* ll loop */

/*   if pos(excstring,translate(fils2.ll))=0 then ndirs=ndirs+1 */
  end 

  aline=aline"; # directories= "ndirs
  totdirs=totdirs+ndirs
  call dosay aline
end
if ndo>1 then do
   todo='==  Total: '
   aline=todo||" # files=" totfils
   aline=aline||" ("||addcomma(totsize)||")"

  aline=aline"; # directories= "totdirs
  call dosay aline
end

if dodirsummary=1 then do
  waslevel=0
   do mm=1 to dirstats.0  /* determine "displayed" names */
      arf=dirstats.mm
      goo=translate(arf,' ','\')
      nlevel=words(goo)
      pud='  '
      if nlevel>2 then pud='  '||copies(subflag,nlevel-2)
      pud=pud||' '||copies(' ',1+(nlevel-1)*2)
      geeble=pud||word(Goo,nlevel)
      if waslevel=0 then  waslevel=nlevel+1
      if nlevel<=waslevel then geeble=arf
      dirstats.mm.!showme=geeble
      dirstats.!showmax=max(dirstats.!showmax,length(geeble))
   end 
  call dosay ' '
  call dosay left("Directory",dirstats.!showmax)||"  #files (bytes) [ Subdirs : #files (bytes) "
   call dosay '     '||copies('-',dirstats.!showmax)

   do mm=1 to dirstats.0                /* subtree sizes, and display */
      arf=dirstats.mm
      arf2=lefT(arf,dirstats.!maxchar)
      files_in=dirstats.arf.!ct ; bytes_in=dirstats.arf.!size
      nsubs=0
      do mm2=mm+1 to dirstats.0
         arf3=dirstats.mm2
         if abbrev(arf3,arf)=0 then leave
         files_in=files_in+dirstats.arf3.!ct
         bytes_in=bytes_in+dirstats.arf3.!size
         nsubs=nsubs+1
      end
      geeble=left(dirstats.mm.!showme,dirstats.!showmax)|| ' '  dirstats.arf.!ct '(' || dirsizeM(dirstats.arf.!size) || ')'
      if nsubs>0 then geeble=geeble||' ['nsubs ': ' || files_in || ' (' ||dirsizeM(bytes_in)||')'
      call dosay geeble
   end 

end 
say
return 1

/************/
/* ADD COMMAS TO A NUMBER */
addcomma:procedure
parse arg aval,ndec
parse var aval p1 '.' p2

if ndec='' then do
   p2=''
end
else do
   p2='.'||left(p2,ndec,'0')
end /* do */

plen=length(p1)
p1new=''
do i=1 to 10000 while plen>3
   p1new=','right(p1,3)||p1new
   p1=delstr(p1,plen-2)
   plen=plen-3
end /* do */

return p1||p1new||p2

/*****************/
dosay:
parse arg a1
if tofile>0 then do
   call lineout outfile,a1
end
if toscreen>0 then do
   say a1
end
return  0

/*********/
/* write html header stuff */
ini_html:

call lineout outfile,'<html><head><title>Results of DIRTREE</title></head>'
call lineout outfile,'<body>'
call lineout outfile,'<h2>DirTree Results </h2>'
call lineout outfile,'<Pre>'
return


/*************/
/* express size as xxx.xM  or xxx.xK */
dirsizeM:procedure
parse arg asize;asize=strip(asize)
if asize<1000 then return asize
if asize<1000000 then do
    asize=asize/1000
   parse var asize a1 '.' a2

    return a1||'.'||lefT(a2,1)||'k'
end
asize=asize/1000000
parse var asize a1 '.' a2
return a1||'.'||lefT(a2,1)||'m'

